package com.godyo.libreoffice.application;

import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.nio.file.Files;
import java.nio.file.Paths;

import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.jface.layout.GridLayoutFactory;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;

public class Application {

	private static LibreOfficePanel panel;

	public static void main(final String[] args) {
		final Display display = new Display();
		final Shell shell = new Shell(display);

		GridLayoutFactory.fillDefaults().applyTo(shell);

		final Button button = new Button(shell, SWT.PUSH);
		button.setText("Reset");
		button.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				openDocument();
			}
		});
		GridDataFactory.fillDefaults().grab(true, false).align(SWT.FILL, SWT.TOP).applyTo(button);

		panel = new LibreOfficePanel(shell);
		panel.setVisible(true);
		final GridLayout layout = (GridLayout) panel.getLayout();
		layout.marginWidth = 10;
		layout.marginHeight = 10;

		shell.addDisposeListener($ -> panel.dispose());

		shell.pack();
		shell.open();
		shell.setSize(1000, 800);

		openDocument();

		while (!shell.isDisposed()) {
			if (!display.readAndDispatch()) {
				display.sleep();
			}
		}
		display.dispose();
	}

	private static String getRTFContent() {
		try {
			final URL resource = Application.class.getClassLoader().getResource("content.rtf");
			final URI uri = resource.toURI();
			return new String(Files.readAllBytes(Paths.get(uri)));
		} catch (final URISyntaxException | IOException e) {
			throw new RuntimeException(e);
		}
	}

	private static void openDocument() {
		panel.loadDocument(new TextFragment(getRTFContent()));
	}

}