package com.godyo.libreoffice.application;

import java.awt.BorderLayout;
import java.awt.Frame;
import java.awt.Panel;

import javax.swing.JApplet;

import org.eclipse.swt.SWT;
import org.eclipse.swt.awt.SWT_AWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;

import com.godyo.libreoffice.unointerface.Connection;
import com.godyo.libreoffice.unointerface.Document;

public class LibreOfficePanel extends Composite {

	private Connection connection;
	private com.godyo.libreoffice.unointerface.Frame officeFrame;
	private Frame awtFrame;
	private Document document;

	public LibreOfficePanel(final Composite parent) {
		super(parent, SWT.EMBEDDED);
		setLayout(new GridLayout());
		setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
	}

	@Override
	public boolean setFocus() {
		if (officeFrame != null) {
			ensureOfficeApplication();
			officeFrame.setFocus();
			return true;
		}
		return super.setFocus();
	}

	public void loadDocument(final TextFragment doc) {
		setVisible(false);
		setEnabled(false);
		try {
			closeDocument();
			officeFrame = activateNewFrame();
			try {
				awtFrame.setVisible(false);
				document = officeFrame.loadDocument(doc.getUrl().toString().replace("file:/", "file:///"));
			} finally {
				awtFrame.setVisible(true);
			}
		} catch (final Exception e) {
			throw new RuntimeException(e);
		} finally {
			setVisible(true);
			setEnabled(true);
		}
	}

	private void closeDocument() {
		if (document != null) {
			document.close();
			document = null;
		}
	}

	private com.godyo.libreoffice.unointerface.Frame activateNewFrame() {
		try {
			setRedraw(false);

			if (awtFrame != null) {
				awtFrame.dispose();
			}

			final Panel panel = new JApplet();
			panel.setLayout(new BorderLayout());

			awtFrame = SWT_AWT.new_Frame(this);
			awtFrame.add(panel);

			ensureOfficeApplication();

			final com.godyo.libreoffice.unointerface.Frame newFrame = connection.getOfficeFrame(panel);

			layout();

			return newFrame;
		} finally {
			setRedraw(true);
		}
	}

	private void ensureOfficeApplication() {
		if (connection == null) {
			connection = new Connection();
		}
	}
}