
package com.godyo.libreoffice.unointerface;

import java.awt.Container;
import java.util.Arrays;

import com.godyo.libreoffice.application.Configuration;
import com.sun.star.awt.XSystemChildFactory;
import com.sun.star.awt.XToolkit;
import com.sun.star.awt.XWindow;
import com.sun.star.awt.XWindowPeer;
import com.sun.star.comp.beans.ContainerFactory;
import com.sun.star.comp.beans.OfficeConnection;
import com.sun.star.comp.beans.OfficeWindow;
import com.sun.star.comp.helper.BootstrapException;
import com.sun.star.frame.XFrame;
import com.sun.star.frame.XFrames;
import com.sun.star.frame.XFramesSupplier;
import com.sun.star.lang.XEventListener;
import com.sun.star.lang.XMultiComponentFactory;
import com.sun.star.lang.XMultiServiceFactory;
import com.sun.star.uno.UnoRuntime;
import com.sun.star.uno.XComponentContext;

import ag.ion.bion.officelayer.NativeView;
import ooo.connector.BootstrapSocketConnector;
import ooo.connector.server.OOoServer;

public class Connection implements OfficeConnection {

	private static final String[] ARGUMENTS = new String[] {
			"--nologo",
			"--nodefault",
			"--norestore",
			"--nocrashreport",
			"--nolockcheck",
			"--invisible" };

	static {
		System.loadLibrary("jawt");
	}

	private XComponentContext context = null;

	public Connection() {
		context = connect();
	}

	public Frame getOfficeFrame(final Container container) {
		try {
			final NativeView nativeView = new NativeView();
			container.add(nativeView);

			final XMultiComponentFactory serviceManager = context.getServiceManager();
			final XMultiServiceFactory xMultiServiceFactory = UnoRuntime.queryInterface(XMultiServiceFactory.class,
					serviceManager);

			final Object toolkit = xMultiServiceFactory.createInstance("com.sun.star.awt.Toolkit");
			final XToolkit xToolkit = UnoRuntime.queryInterface(XToolkit.class, toolkit);

			final XSystemChildFactory xChildFactory = UnoRuntime.queryInterface(XSystemChildFactory.class, xToolkit);

			final Integer handle = nativeView.getHWND();
			final short systemType = (short) nativeView.getNativeWindowSystemType();
			final XWindowPeer xWindowPeer = xChildFactory.createSystemChild(handle, new byte[0], systemType);

			final XWindow xWindow = UnoRuntime.queryInterface(XWindow.class, xWindowPeer);

			final Object frame = xMultiServiceFactory.createInstance("com.sun.star.frame.Frame"); //$NON-NLS-1$
			final XFrame xFrame = UnoRuntime.queryInterface(XFrame.class, frame);

			xFrame.initialize(xWindow);

			final Object desktop = xMultiServiceFactory.createInstance("com.sun.star.frame.Desktop"); //$NON-NLS-1$
			final XFramesSupplier xFramesSupplier = UnoRuntime.queryInterface(XFramesSupplier.class, desktop);

			final XFrames xFrames = xFramesSupplier.getFrames();
			xFrames.append(xFrame);

			return new Frame(xFrame);
		} catch (final Exception exception) {
			throw new RuntimeException(exception);
		}
	}

	private XComponentContext connect() {
		try {
			final OOoServer oooServer = new OOoServer(Configuration.LIBRE_OFFICE_PATH, Arrays.asList(ARGUMENTS));
			return new BootstrapSocketConnector(oooServer).connect();
		} catch (final BootstrapException e) {
			throw new RuntimeException(e);
		}
	}

	@Override
	public void addEventListener(final XEventListener eventListener) {
	}

	@Override
	public void dispose() {
	}

	@Override
	public OfficeWindow createOfficeWindow(final Container arg0) {
		throw new UnsupportedOperationException();
	}

	@Override
	synchronized public XComponentContext getComponentContext() {
		throw new UnsupportedOperationException();
	}

	@Override
	public void removeEventListener(final XEventListener eventListener) {
	}

	@Override
	public void setContainerFactory(final ContainerFactory containerFactory) {
	}

	@Override
	public void setUnoUrl(final String url) {
	}

}